texture g_txSrcColor;

sampler2D g_samSrcColor =
sampler_state
{
    Texture = <g_txSrcColor>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
};

static const int g_cKernelSize = 8;

float2 PixelKernel_H[g_cKernelSize] =
{
    { 0, 0},
    { 0, 1},
    { 0, 2},
    { 0, 3},
    { 0, 4},
    { 0, 5},
    { 0, 6},
    { 0, 7}
};

float2 TexelKernel_H[g_cKernelSize]
<
    string ConvertPixelsToTexels = "PixelKernel_H";
>;

float2 PixelKernel_V[g_cKernelSize] =
{
    { 0, 0},
    { 1, 0},
    { 2, 0},
    { 3, 0},
    { 4, 0},
    { 5, 0},
    { 6, 0},
    { 7, 0}
};

float2 TexelKernel_V[g_cKernelSize]
<
    string ConvertPixelsToTexels = "PixelKernel_V";
>;

static const float BlurWeights[g_cKernelSize] = 
{
    0.150,
    0.135,
    0.100,
    0.065,
    0.025,
    0.015,
    0.007,
    0.003
};

float3 LuminanceConv = {0.9, 1.5, 0.6};
float BloomShift = 0.75;
//float BloomShift = 1.5;
float BloomScale = 2.5;

float4 PostProcessPS0(float2 Tex : TEXCOORD0) : COLOR0
{
    return float4((dot(tex2D(g_samSrcColor, Tex), LuminanceConv) - BloomShift) * BloomScale, 0, 0, 0);
}

float4 PostProcessPS1(float2 Tex : TEXCOORD0) : COLOR0
{
    float Light = 0;

    for (int i = 0; i < g_cKernelSize; i++) Light += max(0, tex2D(g_samSrcColor, Tex - TexelKernel_V[i]).r) * BlurWeights[i];
    for (int i = 1; i < g_cKernelSize; i++) Light += max(0, tex2D(g_samSrcColor, Tex + TexelKernel_V[i]).r) * BlurWeights[i];

    return float4(Light, 0, 0, 0);
}

float4 PostProcessPS2(float2 Tex : TEXCOORD0) : COLOR0
{
    float Light = 0;

    for (int i = 0; i < g_cKernelSize; i++) Light += max(0, tex2D(g_samSrcColor, Tex - TexelKernel_H[i]).r) * BlurWeights[i];
    for (int i = 1; i < g_cKernelSize; i++) Light += max(0, tex2D(g_samSrcColor, Tex + TexelKernel_H[i]).r) * BlurWeights[i];

    return float4(Light, 0, 0, 0);
}

float4 PostProcessPS3(float2 Tex : TEXCOORD0) : COLOR0
{
    float Light = tex2D(g_samSrcColor, Tex).r;

    return float4(Light, Light, Light, 1);
}

technique PostProcess
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS0();
        ZEnable = false;
    }
    pass p1
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS1();
        ZEnable = false;
    }
    pass p2
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS2();
        ZEnable = false;
    }
    pass p3
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS3();
        ZEnable = false;
    }
}
