texture g_txSrcColor;

sampler2D g_samSrcColor =
sampler_state
{
    Texture = <g_txSrcColor>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
//    MinFilter = Linear;
//    MagFilter = Linear;
    MipFilter = None;
};

texture g_txSrcLight;

sampler2D g_samSrcLight =
sampler_state
{
    Texture = <g_txSrcLight>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
};

static const float2 TexOffset[16] =
{
    {-0.5,-0.5},
    { 0.5,-0.5},
    {-0.5, 0.5},
    { 0.5, 0.5},
    {-1.5,-1.5},
    {-0.5,-1.5},
    { 0.5,-1.5},
    { 1.5,-1.5},
    {-1.5,-0.5},
    { 1.5,-0.5},
    {-1.5, 0.5},
    { 1.5, 0.5},
    {-1.5, 1.5},
    {-0.5, 1.5},
    { 0.5, 1.5},
    { 1.5, 1.5},
};

float3 LuminanceConv = {0.9, 1.5, 0.6};
float TexScale = 1;
float AdaptTime = 0;
float IncDelay = 1.0;
float DecDelay = 5.0;
float MaxLight = 0.5;
float MinLight = 2.5;

float4 PostProcessPS0(float2 Tex : TEXCOORD0) : COLOR0
{
    float Light = 0;

    for (int i = 0; i < 4; i++) Light += pow(dot((float3)tex2D(g_samSrcColor, Tex + (TexOffset[i] * TexScale)), LuminanceConv), 2);

    return float4(Light / 4, 0, 0, 0);
}

float4 PostProcessPS1(float2 Tex : TEXCOORD0) : COLOR0
{
    float Light = 0;

    for (int i = 0; i < 16; i++) Light += tex2D(g_samSrcColor, Tex + (TexOffset[i] * TexScale)).r;

    return float4(Light / 16, 0, 0, 0);
}

float4 PostProcessPS2(float2 Tex : TEXCOORD0) : COLOR0
{
    float Light = 0;
    float Write = tex2D(g_samSrcLight, float2(0.5, 0.5)).r;

    for (int i = 0; i < 4; i++) Light += tex2D(g_samSrcColor, Tex + (TexOffset[i] * TexScale)).r;

    Light /= 4;
    Light = sqrt(Light);
    Light *= 3;
//    Light *= 8;

    if (Write > Light)
    {
        if (IncDelay != 0)
        {
            Write -= AdaptTime / IncDelay;
            Write = max(Light, Write);
        } else
        {
            Write = Light;
        }
    }
    if (Write < Light)
    {
        if (DecDelay != 0)
        {
            Write += AdaptTime / DecDelay;
            Write = min(Light, Write);
        } else
        {
            Write = Light;
        }
    }
    Write = max(MaxLight, min(MinLight, Write));

    return float4(Write, 0, 0, 0);
}

float4 PostProcessPS3(float2 Tex : TEXCOORD0) : COLOR0
{
    return float4((float3)tex2D(g_samSrcColor, Tex) / tex2D(g_samSrcLight, float2(0.5, 0.5)).r, 1);
}

technique PostProcess
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS0();
        ZEnable = false;
    }
    pass p1
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS1();
        ZEnable = false;
    }
    pass p2
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS2();
        ZEnable = false;
    }
    pass p3
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 PostProcessPS3();
        ZEnable = false;
    }
}
